// filename ******** IR.C ************** 
// Interrupt Input Capture program for the 9S12C32   
// PT2 is input from IR sensor

//  This example accompanies the books
//   "Embedded Microcomputer Systems: Real Time Interfacing", 
//     Thomsen, copyright (c) 2006,
//   "Introduction to Embedded Microcomputer Systems: 
//    Motorola 6811 and 6812 Simulation", Brooks-Cole, copyright (c) 2002,
//    Jonathan W. Valvano 3/17/07

// You may use, edit, run or distribute this file 
//    as long as the above copyright notices remain

#include <hidef.h>         /* common defines and macros */
#include <mc9s12c32.h>     /* derivative information */

#pragma LINK_INFO DERIVATIVE "mc9s12c32"
unsigned short IR_Period[20]; // resolution = 1us
unsigned short IR_Count;  // number of measurements
unsigned short Last;   // time of previous edge

// ------------IR_Init--------------
// enable input capture 2 interrupt on falling edge
// inputs: none
// outputs: none
// assumes 1us TCNT 
void IR_Init(void){  
asm sei           // make atomic
  DDRT &= ~0x04;  // PT2 input
  TCTL4 |= 0x20;   
  TCTL4 &= ~0x10;  // falling edge IC2
  TIE |= 0x04;     // Arm IC2
  IR_Count = 0;  
  Last = TCNT;     // first measurement wrong
  TIOS &= ~0x04;   // PT2 input capture
asm cli
}
void interrupt 10 IC2Han(void){ 
  TFLG1 = 0x04;     // clear C2F
  IR_Period[IR_Count] = TC2-Last;
  Last = TC2;
  if(IR_Count<19){
    IR_Count ++;
  }
}

void main(void) {	
  TSCR1 = 0x80;   // enable TCNT, 1us
  TSCR2 = 0x02;   // divide by 4 TCNT prescale, TOI disarm
  PACTL = 0;      // timer prescale used for TCNT
  IR_Init(); // start IC thread
  for(;;) {	
  }
}
